#!/usr/bin/wish
#***********************************************************************
#                  Tcl/Tk-GUI fuer gnuplot
#                  =======================
#                     Uwe Berger; 2010
#
# Voraussetzungen: sqlite3
#                  gnuplot
#
# Datenbasis ist eine sqlite-DB "temperatur" mit folgender Tabelle:
# 
# sqlite> .schema temperatur
# CREATE TABLE temperatur (timestamp text, temp real);
# CREATE INDEX timestamp_index on temperatur (timestamp);
#
# Beispielsdatensaetze:
# 2010-11-16T21:14:00|12.0
# 2010-11-16T21:15:00|13.0
# 2010-11-16T21:16:00|12.5
#
#
# ---------
# Have fun!
#
#***********************************************************************

set gvar(dx) 800
set gvar(dy) 500

set gvar(refresh) 1
set gvar(zyklus) 60000

set gvar(ymin) -10
set gvar(ymax) 40

set gvar(begin) 0
set gvar(dt) 24

set gvar(day)    [clock format [clock seconds] -format %d]
set gvar(month)  [clock format [clock seconds] -format %m]
set gvar(year)   [clock format [clock seconds] -format %Y]
set gvar(hour)   [clock format [clock seconds] -format %H]
set gvar(minute) [clock format [clock seconds] -format %M]

set gvar(ytics) 5
set gvar(sql) "select * from temperatur"

set temp_cal(month)	0
set temp_cal(year)	0
set temp_cal(day)	0
set temp_cal(win)   ""


#*****************
proc set_display_date {} {
	global gvar
	set gvar(date_display) "Basiszeitpunkt: $gvar(day).$gvar(month).$gvar(year) $gvar(hour):$gvar(minute)"
}


#*****************
proc calendar {month year win} {
	global temp_cal
	# alte Widgets loeschen, wenn bereits vorhanden...
	catch {
		foreach w [grid slaves .cfg.cal.fmy] {destroy $w}
		destroy .cfg.cal.fmy
		foreach w [grid slaves .cfg.cal.fd] {destroy $w}
		destroy .cfg.cal.fd
	}
	# ein paar Variablen
	set days {0 31 28 31 30 31 30 31 31 30 31 30 31}
	set day_name {Mo Di Mi Do Fr Sa So}
	set temp_cal(month) $month
	set temp_cal(year) $year
	set temp_cal(win) $win
	# Schaltjahr
	if {($month == 2) && ([expr $year % 4] == 0) && ([expr $year % 100] != 0) || ([expr $year % 400] == 0)} {
		set leap_year 1
	} else {
		set leap_year 0;
	}
	# Monat/Jahr
	frame .cfg.cal.fmy 
	pack  .cfg.cal.fmy  -in $win
	label .cfg.cal.fmy.mprev -text "<<  "
	label .cfg.cal.fmy.mnext -text "  >>"
	label .cfg.cal.fmy.my -text "$month/$year"
	grid  .cfg.cal.fmy.mprev -row 0 -column 0
	grid  .cfg.cal.fmy.my    -row 0 -column 1
	grid  .cfg.cal.fmy.mnext -row 0 -column 2
	bind  .cfg.cal.fmy.mprev <Enter> {%W configure -bg yellow}
	bind  .cfg.cal.fmy.mprev <Leave> {%W configure -bg gray85}
	bind  .cfg.cal.fmy.mprev <ButtonPress> {
								incr temp_cal(month) -1
								if {$temp_cal(month)<1} {
											incr temp_cal(year) -1
											set temp_cal(month) 12
											}
								calendar $temp_cal(month) $temp_cal(year) $temp_cal(win)
							}
	bind .cfg.cal.fmy.mnext <Enter> {%W configure -bg yellow}
	bind .cfg.cal.fmy.mnext <Leave> {%W configure -bg gray85}
	bind .cfg.cal.fmy.mnext <ButtonPress> {
								incr temp_cal(month) 
								if {$temp_cal(month)>12} {
											incr temp_cal(year)
											set temp_cal(month) 1
											}
								calendar $temp_cal(month) $temp_cal(year) $temp_cal(win)
							}
	# Kalenderblatt
	frame .cfg.cal.fd
	pack  .cfg.cal.fd
	# ...Wochentagnamen
	for {set i 0} {$i < 9} {incr i} {
		label .cfg.cal.fd.dn$i -text [lindex $day_name $i]
		grid  .cfg.cal.fd.dn$i -row 0 -column $i
	}
	# ...Tage
	set row 1
	set col [clock format [clock scan "$month/01/$year"] -format %w]
	set col [expr $col-1]
	if {$col < 0} {set col 6}
	for {set i 1} {$i < [expr [lindex $days $month] + 1 + $leap_year]} {incr i} {
		label .cfg.cal.fd.d$i -text " $i "
		grid  .cfg.cal.fd.d$i -row $row -column $col
		bind  .cfg.cal.fd.d$i <Enter> {%W configure -bg yellow}
		bind  .cfg.cal.fd.d$i <Leave> {%W configure -bg gray85}
		bind  .cfg.cal.fd.d$i <ButtonPress> {
									set temp_cal(day) [string trim [%W cget -text]]
									set gvar(year) "$temp_cal(year)"
									set gvar(month) "$temp_cal(month)"
									set gvar(day) "$temp_cal(day)"
									set_display_date
									}
		incr col
		if {$col > 6} {
			set col 0
			incr row
		}
	}
}


#*****************
proc gui_init {} {
	global gvar
	wm title . "gnuplot-Diagramm"
	frame .top
	pack .top
	button .quit -text Konfiguration -command {catch {config_dialog} {} }
	pack .quit -in .top
	canvas .g -width $gvar(dx) -height $gvar(dy) -bg white
	pack .g -in .top
}


#*****************
proc config_dialog {} {
	global gvar
	toplevel .cfg
	wm title .cfg "Konfiguration"

	# ******* Refresh-Zyklus ********
	checkbutton .cfg.cb_refresh -text "zykl. Refresh \[s\]"\
				-variable gvar(refresh)\
				-command {	if {$gvar(refresh)} {
									.cfg.sb_zyklus configure -state normal
								}\
							else {
									.cfg.sb_zyklus configure -state disabled
								}
						}
	spinbox .cfg.sb_zyklus -textvariable v -from 1 -to 120 -increment 1\
				-width 4\
				-command {set gvar(zyklus) [expr $v * 1000]}
	.cfg.sb_zyklus set [expr $gvar(zyklus)/1000]
	grid .cfg.cb_refresh -sticky w -row 0 -column 0
	grid .cfg.sb_zyklus            -row 0 -column 1

	# ********* dx/dy *********
	label .cfg.l_dxdy -text "Diagrammgroesse dx/dy"
	spinbox .cfg.sb_dx -textvariable dx -from 10 -to 1000 -increment 10\
				-width 4\
				-command {set gvar(dx) $dx}
	.cfg.sb_dx set $gvar(dx)
	spinbox .cfg.sb_dy -textvariable dy -from 10 -to 1000 -increment 10\
				-width 4\
				-command {set gvar(dy) $dy}
	.cfg.sb_dy set $gvar(dy)
	grid  .cfg.l_dxdy -sticky w -row 2 -column 0
	grid  .cfg.sb_dx              -row 2 -column 1
	grid  .cfg.sb_dy              -row 2 -column 2
	
	# ********* Min-/Max-Temperatur *********
	label .cfg.l_temp_bereich -text "Temperaturbereich von/bis"
	spinbox .cfg.sb_min_temp -textvariable min_t -from -50 -to 100 -increment 5\
				-width 4\
				-command {set gvar(ymin) $min_t}
	.cfg.sb_min_temp set $gvar(ymin)
	spinbox .cfg.sb_max_temp -textvariable max_t -from -50 -to 100 -increment 5\
				-width 4\
				-command {set gvar(ymax) $max_t}
	.cfg.sb_max_temp set $gvar(ymax)
	grid  .cfg.l_temp_bereich -sticky w -row 3 -column 0
	grid  .cfg.sb_min_temp              -row 3 -column 1
	grid  .cfg.sb_max_temp              -row 3 -column 2
	
	# ********* Basis-Zeitpunkt ************
	set_display_date
	label .cfg.l_basis_date -textvariable gvar(date_display)
	grid  .cfg.l_basis_date              -row 4 -column 0 -columnspan 3 -pady 15
	
	# ********* folgender Zeitpunkt ist Anfang oder Ende der Zeitachse *********
	checkbutton .cfg.cb_begin -text "...Datum/Zeit ist xmin"\
				-variable gvar(begin)
	grid .cfg.cb_begin -sticky w -row 8 -column 0
	
	# ********* Uhrzeit *************
	label .cfg.l_time -text "Uhrzeit \[h/m\]"
	spinbox .cfg.sb_time_hour -textvariable hour_t -from 0 -to 23 -increment 1\
				-width 4\
				-command {set gvar(hour) $hour_t}
	.cfg.sb_time_hour set $gvar(hour)
	spinbox .cfg.sb_time_minute -textvariable minute_t -from 0 -to 59 -increment 1\
				-width 4\
				-command {set gvar(minute) $minute_t}
	.cfg.sb_time_minute set $gvar(minute)
	grid  .cfg.l_time -sticky w -row 6 -column 0
	grid  .cfg.sb_time_hour     -row 6 -column 1
	grid  .cfg.sb_time_minute   -row 6 -column 2	

	bind  .cfg.sb_time_hour   <ButtonPress> {set_display_date}	
	bind  .cfg.sb_time_minute <ButtonPress> {set_display_date}	
		
	# ********* Anzeigebereich Zeitachse *********
	label .cfg.l_dt -text "Anzeigebereich \[h\]"
	spinbox .cfg.sb_dt -textvariable dt -from 1 -to 1000 -increment 1\
				-width 4\
				-command {set gvar(dt) $dt}
	.cfg.sb_dt set $gvar(dt)
	grid  .cfg.l_dt -sticky w -row 7 -column 0
	grid  .cfg.sb_dt          -row 7 -column 1
	
	# ******** Kalenderblatt **********
	labelframe .cfg.cal -text "Datumsauswahl"
	grid .cfg.cal -row 5 -column 0 -columnspan 3 
	calendar $gvar(month) $gvar(year) .cfg.cal

	# ********* Uebernehmen-Button **********
	button .cfg.b_confirm -text Anzeigen... -command {
						after cancel $gvar(zyklus_id)
						.g configure -width $gvar(dx) -height $gvar(dy)
						set_display_date
						generate_plot
									}
	grid  .cfg.b_confirm -row 9 -column 0 -columnspan 3
	
}


#*****************
proc generate_plot {} {
	global gvar
	# xmin/ymin setzen
	set temp_d [clock scan "$gvar(year)-$gvar(month)-$gvar(day)T$gvar(hour):$gvar(minute):00" -format %Y-%m-%dT%H:%M:%S]
	if {$gvar(begin)&& !$gvar(refresh)} {
	    set gvar(xmin) "$gvar(year)-$gvar(month)-$gvar(day)T$gvar(hour):$gvar(minute):00"
    	set gvar(xmax) [clock format [expr $temp_d + $gvar(dt)*3600] -format %Y-%m-%dT%H:%M:%S]
    } else {
	    set gvar(xmax) "$gvar(year)-$gvar(month)-$gvar(day)T$gvar(hour):$gvar(minute):00"
    	set gvar(xmin) [clock format [expr $temp_d - $gvar(dt)*3600] -format %Y-%m-%dT%H:%M:%S]
	}
	# gnuplot oeffnen...	
	set pid [open "| gnuplot" "w"]	
	puts $pid "set ytics $gvar(ytics)"
	puts $pid "set xdata time"
	puts $pid "set timefmt '%Y-%m-%dT%H:%M:%S'"
	puts $pid "set format x '%H.%M\\n%d.%m'"
	puts $pid "set autoscale y"
	puts $pid "set autoscale x"
	puts $pid "set xrange \['$gvar(xmin)':'$gvar(xmax)'\]"
	puts $pid "set yrange \[$gvar(ymin):$gvar(ymax)\]"
	puts $pid "set xlabel 'Zeit'"
	puts $pid "set ylabel 'Temperatur'"
	puts $pid "set datafile separator '|'"
	puts $pid "set grid ytics xtics"
	puts $pid "set terminal tkcanvas size $gvar(dx),$gvar(dy)"
	puts $pid "set output 'canvas.tk'"
	if {[catch {puts $pid "plot \"< sqlite3 temperatur '$gvar(sql)'\" using 1:2 index 0 title 'Sensor 1' with lines"}]} {
		# Fehler bein Zeichnen in gnuplot
		puts "gnuplot-Fehler...!"
		close $pid
	} else {
		close $pid
		source canvas.tk
		exec rm canvas.tk
		gnuplot .g
	}
	if {$gvar(refresh)} {
		set gvar(day)    [clock format [clock seconds] -format %d]
		set gvar(month)  [clock format [clock seconds] -format %m]
		set gvar(year)   [clock format [clock seconds] -format %Y]
		set gvar(hour)   [clock format [clock seconds] -format %H]
		set gvar(minute) [clock format [clock seconds] -format %M]
		set_display_date
		set gvar(zyklus_id) [after $gvar(zyklus) generate_plot]
	}
}

gui_init
generate_plot	


